@@ -1,9 +1,9 @@
 /*
- * "$Id: serial.c,v 1.41 2002/02/12 19:05:37 mike Exp $"
+ * "$Id: serial.c 5400 2006-04-14 19:04:02Z mike $"
  *
  *   Serial port backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2006 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,12 +15,14 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
+ *   This file is subject to the Apple OS-Developed Software exception.
+ *
  * Contents:
  *
  *   main()         - Send a file to the printer or server.
@@ -31,6 +33,7 @@
  * Include necessary headers.
  */
 
+#include <cups/backend.h>
 #include <cups/cups.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -48,6 +51,11 @@
 #  include <unistd.h>
 #  include <fcntl.h>
 #  include <termios.h>
+#  ifdef __hpux
+#    include <sys/time.h>
+#  else
+#    include <sys/select.h>
+#  endif /* __hpux */
 #  ifdef HAVE_SYS_IOCTL_H
 #    include <sys/ioctl.h>
 #  endif /* HAVE_SYS_IOCTL_H */
@@ -72,6 +80,18 @@
 #  endif /* CNEW_RTSCTS */
 #endif /* !CRTSCTS */
 
+#if defined(__APPLE__)
+#  include <CoreFoundation/CoreFoundation.h>
+#  include <IOKit/IOKitLib.h>
+#  include <IOKit/serial/IOSerialKeys.h>
+#  include <IOKit/IOBSD.h>
+#endif /* __APPLE__ */
+
+#if defined(__linux) && defined(TIOCGSERIAL)
+#  include <linux/serial.h>
+#  include <linux/ioctl.h>
+#endif /* __linux && TIOCGSERIAL */
+
 
 /*
  * Local functions...
@@ -88,33 +108,36 @@ void	list_devices(void);
  *    printer-uri job-id user title copies options [file]
  */
 
-int			/* O - Exit status */
-main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
-     char *argv[])	/* I - Command-line arguments */
+int					/* O - Exit status */
+main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
+     char *argv[])			/* I - Command-line arguments */
 {
-  char		method[255],	/* Method in URI */
-		hostname[1024],	/* Hostname */
-		username[255],	/* Username info (not used) */
-		resource[1024],	/* Resource info (device and options) */
-		*options,	/* Pointer to options */
-		name[255],	/* Name of option */
-		value[255],	/* Value of option */
-		*ptr;		/* Pointer into name or value */
-  int		port;		/* Port number (not used) */
-  FILE		*fp;		/* Print file */
-  int		copies;		/* Number of copies to print */
-  int		fd;		/* Parallel device */
-  int		wbytes;		/* Number of bytes written */
-  size_t	nbytes,		/* Number of bytes read */
-		tbytes;		/* Total number of bytes written */
-  int		dtrdsr;		/* Do dtr/dsr flow control? */
-  int		bufsize;	/* Size of output buffer for writes */
-  char		buffer[8192],	/* Output buffer */
-		*bufptr;	/* Pointer into buffer */
-  struct termios opts;		/* Serial port options */
-  struct termios origopts;	/* Original port options */
+  char		method[255],		/* Method in URI */
+		hostname[1024],		/* Hostname */
+		username[255],		/* Username info (not used) */
+		resource[1024],		/* Resource info (device and options) */
+		*options,		/* Pointer to options */
+		name[255],		/* Name of option */
+		value[255],		/* Value of option */
+		*ptr;			/* Pointer into name or value */
+  int		port;			/* Port number (not used) */
+  int		fp;			/* Print file */
+  int		copies;			/* Number of copies to print */
+  int		fd;			/* Parallel device */
+  int		rbytes;			/* Number of bytes read */
+  int		wbytes;			/* Number of bytes written */
+  size_t	nbytes,			/* Number of bytes read */
+		tbytes;			/* Total number of bytes written */
+  int		dtrdsr;			/* Do dtr/dsr flow control? */
+  int		bufsize;		/* Size of output buffer for writes */
+  char		buffer[8192],		/* Output buffer */
+		*bufptr;		/* Pointer into buffer */
+  struct termios opts;			/* Serial port options */
+  struct termios origopts;		/* Original port options */
+  fd_set	input,			/* Input set for select() */
+		output;			/* Output set for select() */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
-  struct sigaction action;	/* Actions for POSIX signals */
+  struct sigaction action;		/* Actions for POSIX signals */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
@@ -124,6 +147,20 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
   setbuf(stderr, NULL);
 
+ /*
+  * Ignore SIGPIPE signals...
+  */
+
+#ifdef HAVE_SIGSET
+  sigset(SIGPIPE, SIG_IGN);
+#elif defined(HAVE_SIGACTION)
+  memset(&action, 0, sizeof(action));
+  action.sa_handler = SIG_IGN;
+  sigaction(SIGPIPE, &action, NULL);
+#else
+  signal(SIGPIPE, SIG_IGN);
+#endif /* HAVE_SIGSET */
+
  /*
   * Check command-line...
   */
@@ -131,12 +168,12 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   if (argc == 1)
   {
     list_devices();
-    return (0);
+    return (CUPS_BACKEND_OK);
   }
   else if (argc < 6 || argc > 7)
   {
     fputs("Usage: serial job-id user title copies options [file]\n", stderr);
-    return (1);
+    return (CUPS_BACKEND_FAILED);
   }
 
  /*
@@ -146,7 +183,7 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
   if (argc == 6)
   {
-    fp     = stdin;
+    fp     = 0;
     copies = 1;
   }
   else
@@ -155,10 +192,10 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
     * Try to open the print file...
     */
 
-    if ((fp = fopen(argv[6], "rb")) == NULL)
+    if ((fp = open(argv[6], O_RDONLY)) < 0)
     {
       perror("ERROR: unable to open print file");
-      return (1);
+      return (CUPS_BACKEND_FAILED);
     }
 
     copies = atoi(argv[4]);
@@ -168,7 +205,10 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   * Extract the device name and options from the URI...
   */
 
-  httpSeparate(argv[0], method, username, hostname, &port, resource);
+  httpSeparateURI(HTTP_URI_CODING_ALL, cupsBackendDeviceURI(argv),
+                  method, sizeof(method), username, sizeof(username),
+		  hostname, sizeof(hostname), &port,
+		  resource, sizeof(resource));
 
  /*
   * See if there are any options...
@@ -188,10 +228,33 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   * Open the serial port device...
   */
 
+  fputs("STATE: +connecting-to-device\n", stderr);
+
   do
   {
     if ((fd = open(resource, O_WRONLY | O_NOCTTY | O_EXCL | O_NDELAY)) == -1)
     {
+      if (getenv("CLASS") != NULL)
+      {
+       /*
+        * If the CLASS environment variable is set, the job was submitted
+	* to a class and not to a specific queue.  In this case, we want
+	* to abort immediately so that the job can be requeued on the next
+	* available printer in the class.
+	*/
+
+        fputs("INFO: Unable to open serial port, queuing on next printer in class...\n",
+	      stderr);
+
+       /*
+        * Sleep 5 seconds to keep the job from requeuing too rapidly...
+	*/
+
+	sleep(5);
+
+        return (CUPS_BACKEND_FAILED);
+      }
+
       if (errno == EBUSY)
       {
         fputs("INFO: Serial port busy; will retry in 30 seconds...\n", stderr);
@@ -201,12 +264,14 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
       {
 	fprintf(stderr, "ERROR: Unable to open serial port device file \"%s\": %s\n",
 	        resource, strerror(errno));
-	return (1);
+	return (CUPS_BACKEND_FAILED);
       }
     }
   }
   while (fd < 0);
 
+  fputs("STATE: -connecting-to-device\n", stderr);
+
  /*
   * Set any options provided...
   */
@@ -228,7 +293,8 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
       */
 
       for (ptr = name; *options && *options != '=';)
-        *ptr++ = *options++;
+        if (ptr < (name + sizeof(name) - 1))
+          *ptr++ = *options++;
       *ptr = '\0';
 
       if (*options == '=')
@@ -239,8 +305,9 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
         options ++;
 
-	for (ptr = value; *options && *options != '+';)
-          *ptr++ = *options++;
+	for (ptr = value; *options && *options != '+' && *options != '&';)
+          if (ptr < (value + sizeof(value) - 1))
+            *ptr++ = *options++;
 	*ptr = '\0';
 
 	if (*options == '+')
@@ -253,7 +320,7 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
       * Process the option...
       */
 
-      if (strcasecmp(name, "baud") == 0)
+      if (!strcasecmp(name, "baud"))
       {
        /*
         * Set the baud rate...
@@ -291,25 +358,31 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 	      cfsetispeed(&opts, B38400);
 	      cfsetospeed(&opts, B38400);
 	      break;
-#ifdef B57600
+#  ifdef B57600
 	  case 57600 :
 	      cfsetispeed(&opts, B57600);
 	      cfsetospeed(&opts, B57600);
 	      break;
-#endif /* B57600 */
-#ifdef B115200
+#  endif /* B57600 */
+#  ifdef B115200
 	  case 115200 :
 	      cfsetispeed(&opts, B115200);
 	      cfsetospeed(&opts, B115200);
 	      break;
-#endif /* B115200 */
+#  endif /* B115200 */
+#  ifdef B230400
+	  case 230400 :
+	      cfsetispeed(&opts, B230400);
+	      cfsetospeed(&opts, B230400);
+	      break;
+#  endif /* B230400 */
           default :
 	      fprintf(stderr, "WARNING: Unsupported baud rate %s!\n", value);
 	      break;
 	}
 #endif /* B19200 == 19200 */
       }
-      else if (strcasecmp(name, "bits") == 0)
+      else if (!strcasecmp(name, "bits"))
       {
        /*
         * Set number of data bits...
@@ -330,55 +403,90 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 	      break;
 	}
       }
-      else if (strcasecmp(name, "parity") == 0)
+      else if (!strcasecmp(name, "parity"))
       {
        /*
 	* Set parity checking...
 	*/
 
-	if (strcasecmp(value, "even") == 0)
+	if (!strcasecmp(value, "even"))
 	{
 	  opts.c_cflag |= PARENB;
           opts.c_cflag &= ~PARODD;
 	}
-	else if (strcasecmp(value, "odd") == 0)
+	else if (!strcasecmp(value, "odd"))
 	{
 	  opts.c_cflag |= PARENB;
           opts.c_cflag |= PARODD;
 	}
-	else if (strcasecmp(value, "none") == 0)
+	else if (!strcasecmp(value, "none"))
 	  opts.c_cflag &= ~PARENB;
+	else if (!strcasecmp(value, "space"))
+	{
+	 /*
+	  * Note: we only support space parity with 7 bits per character...
+	  */
+
+	  opts.c_cflag &= ~CSIZE;
+          opts.c_cflag |= CS8;
+	  opts.c_cflag &= ~PARENB;
+        }
+	else if (!strcasecmp(value, "mark"))
+	{
+	 /*
+	  * Note: we only support mark parity with 7 bits per character
+	  * and 1 stop bit...
+	  */
+
+	  opts.c_cflag &= ~CSIZE;
+          opts.c_cflag |= CS7;
+	  opts.c_cflag &= ~PARENB;
+          opts.c_cflag |= CSTOPB;
+        }
       }
-      else if (strcasecmp(name, "flow") == 0)
+      else if (!strcasecmp(name, "flow"))
       {
        /*
 	* Set flow control...
 	*/
 
-	if (strcasecmp(value, "none") == 0)
+	if (!strcasecmp(value, "none"))
 	{
-	  opts.c_iflag &= ~(IXON | IXOFF | IXANY);
+	  opts.c_iflag &= ~(IXON | IXOFF);
           opts.c_cflag &= ~CRTSCTS;
 	}
-	else if (strcasecmp(value, "soft") == 0)
+	else if (!strcasecmp(value, "soft"))
 	{
-	  opts.c_iflag |= IXON | IXOFF | IXANY;
+	  opts.c_iflag |= IXON | IXOFF;
           opts.c_cflag &= ~CRTSCTS;
 	}
-	else if (strcasecmp(value, "hard") == 0 ||
-	         strcasecmp(value, "rtscts") == 0)
+	else if (!strcasecmp(value, "hard") ||
+	         !strcasecmp(value, "rtscts"))
         {
-	  opts.c_iflag &= ~(IXON | IXOFF | IXANY);
+	  opts.c_iflag &= ~(IXON | IXOFF);
           opts.c_cflag |= CRTSCTS;
 	}
-	else if (strcasecmp(value, "dtrdsr") == 0)
+	else if (!strcasecmp(value, "dtrdsr"))
 	{
-	  opts.c_iflag &= ~(IXON | IXOFF | IXANY);
+	  opts.c_iflag &= ~(IXON | IXOFF);
           opts.c_cflag &= ~CRTSCTS;
 
 	  dtrdsr = 1;
 	}
       }
+      else if (!strcasecmp(name, "stop"))
+      {
+        switch (atoi(value))
+	{
+	  case 1 :
+	      opts.c_cflag &= ~CSTOPB;
+	      break;
+
+	  case 2 :
+	      opts.c_cflag |= CSTOPB;
+	      break;
+	}
+      }
     }
 
   tcsetattr(fd, TCSANOW, &opts);
@@ -413,14 +521,16 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   if (bufsize > sizeof(buffer))
     bufsize = sizeof(buffer);
 
+  wbytes = 0;
+
   while (copies > 0)
   {
     copies --;
 
-    if (fp != stdin)
+    if (fp != 0)
     {
       fputs("PAGE: 1 1\n", stderr);
-      rewind(fp);
+      lseek(fp, 0, SEEK_SET);
     }
 
     if (dtrdsr)
@@ -454,7 +564,7 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
     }
 
     tbytes = 0;
-    while ((nbytes = fread(buffer, 1, bufsize, fp)) > 0)
+    while ((nbytes = read(fp, buffer, bufsize)) > 0)
     {
      /*
       * Write the print data to the printer...
@@ -465,20 +575,70 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
       while (nbytes > 0)
       {
-	if ((wbytes = write(fd, bufptr, nbytes)) < 0)
-	  if (errno == ENOTTY)
-	    wbytes = write(fd, bufptr, nbytes);
+       /*
+        * See if we are ready to read or write...
+	*/
 
-	if (wbytes < 0)
+        do
 	{
-	  perror("ERROR: Unable to send print file to printer");
-	  break;
+          FD_ZERO(&input);
+	  FD_SET(fd, &input);
+	  FD_ZERO(&output);
+	  FD_SET(fd, &output);
+        }
+	while (select(fd + 1, &input, &output, NULL, NULL) < 0);
+
+        if (FD_ISSET(fd, &input))
+	{
+	 /*
+	  * Read backchannel data...
+	  */
+
+	  if ((rbytes = read(fd, resource, sizeof(resource))) > 0)
+	  {
+	    fprintf(stderr, "DEBUG: Received %d bytes of back-channel data!\n",
+	            rbytes);
+            cupsBackChannelWrite(resource, rbytes, 1.0);
+          }
 	}
 
-	nbytes -= wbytes;
-	bufptr += wbytes;
+        if (FD_ISSET(fd, &output))
+	{
+	 /*
+	  * Write print data...
+	  */
+
+	  if ((wbytes = write(fd, bufptr, nbytes)) < 0)
+	    if (errno == ENOTTY)
+	      wbytes = write(fd, bufptr, nbytes);
+
+	  if (wbytes < 0)
+	  {
+	   /*
+	    * Check for retryable errors...
+	    */
+
+	    if (errno != EAGAIN && errno != EINTR)
+	    {
+	      perror("ERROR: Unable to send print file to printer");
+	      break;
+	    }
+	  }
+	  else
+	  {
+	   /*
+	    * Update count and pointer...
+	    */
+
+	    nbytes -= wbytes;
+	    bufptr += wbytes;
+	  }
+	}
       }
 
+      if (wbytes < 0)
+        break;
+
       if (argc > 6)
 	fprintf(stderr, "INFO: Sending print file, %lu bytes...\n",
 	        (unsigned long)tbytes);
@@ -492,10 +652,10 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   tcsetattr(fd, TCSADRAIN, &origopts);
 
   close(fd);
-  if (fp != stdin)
-    fclose(fp);
+  if (fp != 0)
+    close(fp);
 
-  return (0);
+  return (wbytes < 0 ? CUPS_BACKEND_FAILED : CUPS_BACKEND_OK);
 }
 
 
@@ -506,27 +666,85 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 void
 list_devices(void)
 {
-#if defined(__hpux) || defined(__sgi) || defined(__sun) || defined(__FreeBSD__) || defined(__OpenBSD__)
+#if defined(__hpux) || defined(__sgi) || defined(__sun) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
   static char	*funky_hex = "0123456789abcdefghijklmnopqrstuvwxyz";
-				/* Funky hex numbering used for some devices */
+					/* Funky hex numbering used for some *
+					 * devices                           */
 #endif /* __hpux || __sgi || __sun || __FreeBSD__ || __OpenBSD__ */
 
 #ifdef __linux
-  int	i;		/* Looping var */
-  int	fd;		/* File descriptor */
-  char	device[255];	/* Device filename */
+  int			i, j;		/* Looping vars */
+  int			fd;		/* File descriptor */
+  char			device[255];	/* Device filename */
+#  ifdef TIOCGSERIAL
+  struct serial_struct	serinfo;	/* serial port info */
+#  endif /* TIOCGSERIAL */
 
 
   for (i = 0; i < 100; i ++)
   {
     sprintf(device, "/dev/ttyS%d", i);
+
+    if ((fd = open(device, O_WRONLY | O_NOCTTY | O_NDELAY)) >= 0)
+    {
+#  ifdef TIOCGSERIAL
+     /*
+      * See if this port exists...
+      */
+
+      serinfo.reserved_char[0] = 0;
+
+      if (!ioctl(fd, TIOCGSERIAL, &serinfo))
+      {
+	if (serinfo.type == PORT_UNKNOWN)
+	{
+	 /*
+	  * Nope...
+	  */
+
+	  close(fd);
+	  continue;
+	}
+      }
+#  endif /* TIOCGSERIAL */
+
+      close(fd);
+
+#  if defined(_ARCH_PPC) || defined(powerpc) || defined(__powerpc)
+      printf("serial serial:%s?baud=230400 \"Unknown\" \"Serial Port #%d\"\n",
+             device, i + 1);
+#  else
+      printf("serial serial:%s?baud=115200 \"Unknown\" \"Serial Port #%d\"\n",
+             device, i + 1);
+#  endif /* _ARCH_PPC || powerpc || __powerpc */
+    }
+  }
+
+  for (i = 0; i < 16; i ++)
+  {
+    sprintf(device, "/dev/usb/ttyUSB%d", i);
     if ((fd = open(device, O_WRONLY | O_NOCTTY | O_NDELAY)) >= 0)
     {
       close(fd);
-      printf("serial serial:%s?baud=115200 \"Unknown\" \"Serial Port #%d\"\n",
+      printf("serial serial:%s?baud=230400 \"Unknown\" \"USB Serial Port #%d\"\n",
              device, i + 1);
     }
   }
+
+  for (i = 0; i < 64; i ++)
+  {
+    for (j = 0; j < 8; j ++)
+    {
+      sprintf(device, "/dev/ttyQ%02de%d", i, j);
+      if ((fd = open(device, O_WRONLY | O_NOCTTY | O_NDELAY)) >= 0)
+      {
+        close(fd);
+        printf("serial serial:%s?baud=115200 \"Unknown\" "
+	       "\"Equinox ESP %d Port #%d\"\n",
+               device, i, j + 1);
+      }
+    }
+  }
 #elif defined(__sgi)
   int		i, j, n;	/* Looping vars */
   char		device[255];	/* Device filename */
@@ -638,13 +856,13 @@ list_devices(void)
   {
     sprintf(device, "/dev/cua/%c", 'a' + i);
     if (access(device, 0) == 0)
-#ifdef B115200
+#  ifdef B115200
       printf("serial serial:%s?baud=115200 \"Unknown\" \"Serial Port #%d\"\n",
              device, i + 1);
-#else
+#  else
       printf("serial serial:%s?baud=38400 \"Unknown\" \"Serial Port #%d\"\n",
              device, i + 1);
-#endif /* B115200 */
+#  endif /* B115200 */
   }
 
  /*
@@ -740,7 +958,7 @@ list_devices(void)
       printf("serial serial:%s?baud=38400 \"Unknown\" \"Serial Port #%d\"\n",
              device, i + 1);
   }
-#elif defined(__FreeBSD__) || defined(__OpenBSD__)
+#elif defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__DragonFly__)
   int	i, j;		/* Looping vars */
   int	fd;		/* File descriptor */
   char	device[255];	/* Device filename */
@@ -775,6 +993,14 @@ list_devices(void)
 	printf("serial serial:%s?baud=115200 \"Unknown\" \"Cyclades #%d Serial Port #%d\"\n",
                device, i, j + 1);
       }
+
+      sprintf(device, "/dev/ttyC%d%c", i, funky_hex[j]);
+      if ((fd = open(device, O_WRONLY | O_NOCTTY | O_NDELAY)) >= 0)
+      {
+	close(fd);
+	printf("serial serial:%s?baud=115200 \"Unknown\" \"Cyclades #%d Serial Port #%d\"\n",
+               device, i, j + 1);
+      }
     }
 
  /*
@@ -858,11 +1084,90 @@ list_devices(void)
 	       device, i, j + 1);
       }
     }
+#elif defined(__APPLE__)
+ /*
+  * Standard serial ports on MacOS X...
+  */
 
+  kern_return_t			kernResult;
+  mach_port_t			masterPort;
+  io_iterator_t			serialPortIterator;
+  CFMutableDictionaryRef	classesToMatch;
+  io_object_t			serialService;
+
+
+  kernResult = IOMasterPort(MACH_PORT_NULL, &masterPort);
+  if (KERN_SUCCESS != kernResult)
+    return;
+
+ /*
+  * Serial devices are instances of class IOSerialBSDClient.
+  */
+
+  classesToMatch = IOServiceMatching(kIOSerialBSDServiceValue);
+  if (classesToMatch != NULL)
+  {
+    CFDictionarySetValue(classesToMatch, CFSTR(kIOSerialBSDTypeKey),
+                         CFSTR(kIOSerialBSDRS232Type));
+
+    kernResult = IOServiceGetMatchingServices(masterPort, classesToMatch,
+                                              &serialPortIterator);
+    if (kernResult == KERN_SUCCESS)
+    {
+      while ((serialService = IOIteratorNext(serialPortIterator)))
+      {
+	CFTypeRef	serialNameAsCFString;
+	CFTypeRef	bsdPathAsCFString;
+	char		serialName[128];
+	char		bsdPath[1024];
+	Boolean		result;
+
+
+	serialNameAsCFString =
+	    IORegistryEntryCreateCFProperty(serialService,
+	                                    CFSTR(kIOTTYDeviceKey),
+					    kCFAllocatorDefault, 0);
+	if (serialNameAsCFString)
+	{
+	  result = CFStringGetCString(serialNameAsCFString, serialName,
+	                              sizeof(serialName),
+				      kCFStringEncodingASCII);
+	  CFRelease(serialNameAsCFString);
+
+	  if (result)
+	  {
+	    bsdPathAsCFString =
+	        IORegistryEntryCreateCFProperty(serialService,
+		                                CFSTR(kIOCalloutDeviceKey),
+						kCFAllocatorDefault, 0);
+	    if (bsdPathAsCFString)
+	    {
+	      result = CFStringGetCString(bsdPathAsCFString, bsdPath,
+	                                  sizeof(bsdPath),
+					  kCFStringEncodingASCII);
+	      CFRelease(bsdPathAsCFString);
+
+	      if (result)
+		printf("serial serial:%s?baud=115200 \"Unknown\" \"%s\"\n",
+		       bsdPath, serialName);
+	    }
+	  }
+	}
+
+	IOObjectRelease(serialService);
+      }
+
+     /*
+      * Release the iterator.
+      */
+
+      IOObjectRelease(serialPortIterator);
+    }
+  }
 #endif
 }
 
 
 /*
- * End of "$Id: serial.c,v 1.41 2002/02/12 19:05:37 mike Exp $".
+ * End of "$Id: serial.c 5400 2006-04-14 19:04:02Z mike $".
  */
