@@ -1,9 +1,9 @@
 /*
- * "$Id: image-tiff.c,v 1.20 2002/01/21 22:25:46 mike Exp $"
+ * "$Id: image-tiff.c 4741 2005-10-02 04:25:52Z mike $"
  *
  *   TIFF file routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2002 by Easy Software Products.
+ *   Copyright 1993-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,22 +15,24 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
+ *   This file is subject to the Apple OS-Developed Software exception.
+ *
  * Contents:
  *
- *   ImageReadTIFF() - Read a TIFF image file.
+ *   _cupsImageReadTIFF() - Read a TIFF image file.
  */
 
 /*
  * Include necessary headers...
  */
 
-#include "image.h"
+#include "image-private.h"
 
 #ifdef HAVE_LIBTIFF
 #  include <tiff.h>	/* TIFF image definitions */
@@ -39,17 +41,18 @@
 
 
 /*
- * 'ImageReadTIFF()' - Read a TIFF image file.
+ * '_cupsImageReadTIFF()' - Read a TIFF image file.
  */
 
 int					/* O - Read status */
-ImageReadTIFF(image_t    *img,		/* IO - Image */
-              FILE       *fp,		/* I - Image file */
-              int        primary,	/* I - Primary choice for colorspace */
-              int        secondary,	/* I - Secondary choice for colorspace */
-              int        saturation,	/* I - Color saturation (%) */
-              int        hue,		/* I - Color hue (degrees) */
-	      const ib_t *lut)		/* I - Lookup table for gamma/brightness */
+_cupsImageReadTIFF(
+    cups_image_t    *img,		/* IO - cupsImage */
+    FILE            *fp,		/* I - cupsImage file */
+    cups_icspace_t  primary,		/* I - Primary choice for colorspace */
+    cups_icspace_t  secondary,		/* I - Secondary choice for colorspace */
+    int             saturation,		/* I - Color saturation (%) */
+    int             hue,		/* I - Color hue (degrees) */
+    const cups_ib_t *lut)		/* I - Lookup table for gamma/brightness */
 {
   TIFF		*tif;			/* TIFF file */
   uint32	width, height;		/* Size of image */
@@ -77,8 +80,8 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
 		pstep,			/* Pixel step (= bpp or -2 * bpp) */
 		scanwidth,		/* Width of scanline */
 		r, g, b, k,		/* Red, green, blue, and black values */
-		alpha;			/* Image includes alpha? */
-  ib_t		*in,			/* Input buffer */
+		alpha;			/* cupsImage includes alpha? */
+  cups_ib_t		*in,			/* Input buffer */
 		*out,			/* Output buffer */
 		*p,			/* Pointer into buffer */
 		*scanline,		/* Scanline buffer */
@@ -171,6 +174,12 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
       img->yppi = 128;
     }
 
+    if (img->xppi == 0 || img->yppi == 0)
+    {
+      fputs("ERROR: Bad TIFF resolution.\n", stderr);
+      img->xppi = img->yppi = 128;
+    }
+
     fprintf(stderr, "DEBUG: TIFF resolution = %fx%f, units=%d\n",
             xres, yres, resunit);
     fprintf(stderr, "DEBUG: Stored resolution = %dx%d PPI\n",
@@ -186,6 +195,23 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
   else
     alpha = 0;
 
+ /*
+  * Check the size of the image...
+  */
+
+  if (width == 0 || width > CUPS_IMAGE_MAX_WIDTH ||
+      height == 0 || height > CUPS_IMAGE_MAX_HEIGHT ||
+      (bits != 1 && bits != 2 && bits != 4 && bits != 8) ||
+      samples < 1 || samples > 4)
+  {
+    fprintf(stderr, "ERROR: Bad TIFF dimensions %ux%ux%ux%u!\n",
+            (unsigned)width, (unsigned)height, (unsigned)bits,
+	    (unsigned)samples);
+    TIFFClose(tif);
+    fclose(fp);
+    return (1);
+  }
+
  /*
   * Setup the image size and colorspace...
   */
@@ -195,12 +221,18 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
   if (photometric == PHOTOMETRIC_MINISBLACK ||
       photometric == PHOTOMETRIC_MINISWHITE)
     img->colorspace = secondary;
+  else if (photometric == PHOTOMETRIC_SEPARATED && primary == CUPS_IMAGE_RGB_CMYK)
+    img->colorspace = CUPS_IMAGE_CMYK;
+  else if (primary == CUPS_IMAGE_RGB_CMYK)
+    img->colorspace = CUPS_IMAGE_RGB;
   else
     img->colorspace = primary;
 
-  bpp = ImageGetDepth(img);
+  fprintf(stderr, "DEBUG: img->colorspace = %d\n", img->colorspace);
 
-  ImageSetMaxTiles(img, 0);
+  bpp = cupsImageGetDepth(img);
+
+  cupsImageSetMaxTiles(img, 0);
 
  /*
   * Set the X & Y start and direction according to the image orientation...
@@ -440,35 +472,38 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
             else
               TIFFReadScanline(tif, in, row, 0);
 
-            if (img->colorspace == IMAGE_WHITE)
+            if (img->colorspace == CUPS_IMAGE_WHITE)
 	    {
 	      if (lut)
-	        ImageLut(in, img->xsize, lut);
+	        cupsImageLut(in, img->xsize, lut);
 
-              ImagePutRow(img, 0, y, img->xsize, in);
+              _cupsImagePutRow(img, 0, y, img->xsize, in);
 	    }
             else
             {
 	      switch (img->colorspace)
 	      {
-		case IMAGE_RGB :
-		    ImageWhiteToRGB(in, out, img->xsize);
+		default :
 		    break;
-		case IMAGE_BLACK :
-		    ImageWhiteToBlack(in, out, img->xsize);
+
+		case CUPS_IMAGE_RGB :
+		    cupsImageWhiteToRGB(in, out, img->xsize);
 		    break;
-		case IMAGE_CMY :
-		    ImageWhiteToCMY(in, out, img->xsize);
+		case CUPS_IMAGE_BLACK :
+		    cupsImageWhiteToBlack(in, out, img->xsize);
 		    break;
-		case IMAGE_CMYK :
-		    ImageWhiteToCMYK(in, out, img->xsize);
+		case CUPS_IMAGE_CMY :
+		    cupsImageWhiteToCMY(in, out, img->xsize);
+		    break;
+		case CUPS_IMAGE_CMYK :
+		    cupsImageWhiteToCMYK(in, out, img->xsize);
 		    break;
 	      }
 
 	      if (lut)
-	        ImageLut(out, img->xsize * bpp, lut);
+	        cupsImageLut(out, img->xsize * bpp, lut);
 
-              ImagePutRow(img, 0, y, img->xsize, out);
+              _cupsImagePutRow(img, 0, y, img->xsize, out);
 	    }
           }
         }
@@ -589,35 +624,38 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
             else
               TIFFReadScanline(tif, in, row, 0);
 
-            if (img->colorspace == IMAGE_WHITE)
+            if (img->colorspace == CUPS_IMAGE_WHITE)
 	    {
 	      if (lut)
-	        ImageLut(in, img->ysize, lut);
+	        cupsImageLut(in, img->ysize, lut);
 
-              ImagePutCol(img, x, 0, img->ysize, in);
+              _cupsImagePutCol(img, x, 0, img->ysize, in);
 	    }
             else
             {
 	      switch (img->colorspace)
 	      {
-		case IMAGE_RGB :
-		    ImageWhiteToRGB(in, out, img->ysize);
+		default :
 		    break;
-		case IMAGE_BLACK :
-		    ImageWhiteToBlack(in, out, img->ysize);
+
+		case CUPS_IMAGE_RGB :
+		    cupsImageWhiteToRGB(in, out, img->ysize);
 		    break;
-		case IMAGE_CMY :
-		    ImageWhiteToCMY(in, out, img->ysize);
+		case CUPS_IMAGE_BLACK :
+		    cupsImageWhiteToBlack(in, out, img->ysize);
 		    break;
-		case IMAGE_CMYK :
-		    ImageWhiteToCMYK(in, out, img->ysize);
+		case CUPS_IMAGE_CMY :
+		    cupsImageWhiteToCMY(in, out, img->ysize);
+		    break;
+		case CUPS_IMAGE_CMYK :
+		    cupsImageWhiteToCMYK(in, out, img->ysize);
 		    break;
 	      }
 
 	      if (lut)
-	        ImageLut(out, img->ysize * bpp, lut);
+	        cupsImageLut(out, img->ysize * bpp, lut);
 
-              ImagePutCol(img, x, 0, img->ysize, out);
+              _cupsImagePutCol(img, x, 0, img->ysize, out);
 	    }
           }
         }
@@ -745,35 +783,38 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
 	      }
             }
 
-            if (img->colorspace == IMAGE_RGB)
+            if (img->colorspace == CUPS_IMAGE_RGB)
 	    {
 	      if (lut)
-	        ImageLut(in, img->xsize * 3, lut);
+	        cupsImageLut(in, img->xsize * 3, lut);
 
-              ImagePutRow(img, 0, y, img->xsize, in);
+              _cupsImagePutRow(img, 0, y, img->xsize, in);
 	    }
             else
             {
 	      switch (img->colorspace)
 	      {
-		case IMAGE_WHITE :
-		    ImageRGBToWhite(in, out, img->xsize);
+		default :
 		    break;
-		case IMAGE_BLACK :
-		    ImageRGBToBlack(in, out, img->xsize);
+
+		case CUPS_IMAGE_WHITE :
+		    cupsImageRGBToWhite(in, out, img->xsize);
 		    break;
-		case IMAGE_CMY :
-		    ImageRGBToCMY(in, out, img->xsize);
+		case CUPS_IMAGE_BLACK :
+		    cupsImageRGBToBlack(in, out, img->xsize);
 		    break;
-		case IMAGE_CMYK :
-		    ImageRGBToCMYK(in, out, img->xsize);
+		case CUPS_IMAGE_CMY :
+		    cupsImageRGBToCMY(in, out, img->xsize);
+		    break;
+		case CUPS_IMAGE_CMYK :
+		    cupsImageRGBToCMYK(in, out, img->xsize);
 		    break;
 	      }
 
 	      if (lut)
-	        ImageLut(out, img->xsize * bpp, lut);
+	        cupsImageLut(out, img->xsize * bpp, lut);
 
-              ImagePutRow(img, 0, y, img->xsize, out);
+              _cupsImagePutRow(img, 0, y, img->xsize, out);
 	    }
           }
         }
@@ -882,35 +923,38 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
 	      }
             }
 
-            if (img->colorspace == IMAGE_RGB)
+            if (img->colorspace == CUPS_IMAGE_RGB)
 	    {
 	      if (lut)
-	        ImageLut(in, img->ysize * 3, lut);
+	        cupsImageLut(in, img->ysize * 3, lut);
 
-              ImagePutCol(img, x, 0, img->ysize, in);
+              _cupsImagePutCol(img, x, 0, img->ysize, in);
 	    }
             else
             {
 	      switch (img->colorspace)
 	      {
-		case IMAGE_WHITE :
-		    ImageRGBToWhite(in, out, img->ysize);
+		default :
 		    break;
-		case IMAGE_BLACK :
-		    ImageRGBToBlack(in, out, img->ysize);
+
+		case CUPS_IMAGE_WHITE :
+		    cupsImageRGBToWhite(in, out, img->ysize);
 		    break;
-		case IMAGE_CMY :
-		    ImageRGBToCMY(in, out, img->ysize);
+		case CUPS_IMAGE_BLACK :
+		    cupsImageRGBToBlack(in, out, img->ysize);
 		    break;
-		case IMAGE_CMYK :
-		    ImageRGBToCMYK(in, out, img->ysize);
+		case CUPS_IMAGE_CMY :
+		    cupsImageRGBToCMY(in, out, img->ysize);
+		    break;
+		case CUPS_IMAGE_CMYK :
+		    cupsImageRGBToCMYK(in, out, img->ysize);
 		    break;
 	      }
 
 	      if (lut)
-	        ImageLut(out, img->ysize * bpp, lut);
+	        cupsImageLut(out, img->ysize * bpp, lut);
 
-              ImagePutCol(img, x, 0, img->ysize, out);
+              _cupsImagePutCol(img, x, 0, img->ysize, out);
 	    }
           }
         }
@@ -1028,37 +1072,40 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
               TIFFReadScanline(tif, in, row, 0);
 
             if ((saturation != 100 || hue != 0) && bpp > 1)
-              ImageRGBAdjust(in, img->xsize, saturation, hue);
+              cupsImageRGBAdjust(in, img->xsize, saturation, hue);
 
-            if (img->colorspace == IMAGE_RGB)
+            if (img->colorspace == CUPS_IMAGE_RGB)
 	    {
 	      if (lut)
-	        ImageLut(in, img->xsize * 3, lut);
+	        cupsImageLut(in, img->xsize * 3, lut);
 
-              ImagePutRow(img, 0, y, img->xsize, in);
+              _cupsImagePutRow(img, 0, y, img->xsize, in);
 	    }
             else
             {
 	      switch (img->colorspace)
 	      {
-		case IMAGE_WHITE :
-		    ImageRGBToWhite(in, out, img->xsize);
+		default :
 		    break;
-		case IMAGE_BLACK :
-		    ImageRGBToBlack(in, out, img->xsize);
+
+		case CUPS_IMAGE_WHITE :
+		    cupsImageRGBToWhite(in, out, img->xsize);
 		    break;
-		case IMAGE_CMY :
-		    ImageRGBToCMY(in, out, img->xsize);
+		case CUPS_IMAGE_BLACK :
+		    cupsImageRGBToBlack(in, out, img->xsize);
 		    break;
-		case IMAGE_CMYK :
-		    ImageRGBToCMYK(in, out, img->xsize);
+		case CUPS_IMAGE_CMY :
+		    cupsImageRGBToCMY(in, out, img->xsize);
+		    break;
+		case CUPS_IMAGE_CMYK :
+		    cupsImageRGBToCMYK(in, out, img->xsize);
 		    break;
 	      }
 
 	      if (lut)
-	        ImageLut(out, img->xsize * bpp, lut);
+	        cupsImageLut(out, img->xsize * bpp, lut);
 
-              ImagePutRow(img, 0, y, img->xsize, out);
+              _cupsImagePutRow(img, 0, y, img->xsize, out);
 	    }
           }
         }
@@ -1173,49 +1220,57 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
               TIFFReadScanline(tif, in, row, 0);
 
             if ((saturation != 100 || hue != 0) && bpp > 1)
-              ImageRGBAdjust(in, img->ysize, saturation, hue);
+              cupsImageRGBAdjust(in, img->ysize, saturation, hue);
 
-            if (img->colorspace == IMAGE_RGB)
+            if (img->colorspace == CUPS_IMAGE_RGB)
 	    {
 	      if (lut)
-	        ImageLut(in, img->ysize * 3, lut);
+	        cupsImageLut(in, img->ysize * 3, lut);
 
-              ImagePutCol(img, x, 0, img->ysize, in);
+              _cupsImagePutCol(img, x, 0, img->ysize, in);
             }
 	    else
             {
 	      switch (img->colorspace)
 	      {
-		case IMAGE_WHITE :
-		    ImageRGBToWhite(in, out, img->ysize);
+		default :
 		    break;
-		case IMAGE_BLACK :
-		    ImageRGBToBlack(in, out, img->ysize);
+
+		case CUPS_IMAGE_WHITE :
+		    cupsImageRGBToWhite(in, out, img->ysize);
 		    break;
-		case IMAGE_CMY :
-		    ImageRGBToCMY(in, out, img->ysize);
+		case CUPS_IMAGE_BLACK :
+		    cupsImageRGBToBlack(in, out, img->ysize);
 		    break;
-		case IMAGE_CMYK :
-		    ImageRGBToCMYK(in, out, img->ysize);
+		case CUPS_IMAGE_CMY :
+		    cupsImageRGBToCMY(in, out, img->ysize);
+		    break;
+		case CUPS_IMAGE_CMYK :
+		    cupsImageRGBToCMYK(in, out, img->ysize);
 		    break;
 	      }
 
 	      if (lut)
-	        ImageLut(out, img->ysize * bpp, lut);
+	        cupsImageLut(out, img->ysize * bpp, lut);
 
-              ImagePutCol(img, x, 0, img->ysize, out);
+              _cupsImagePutCol(img, x, 0, img->ysize, out);
 	    }
           }
         }
         break;
 
     case PHOTOMETRIC_SEPARATED :
+        inkset  = INKSET_CMYK;
+        numinks = 4;
+
+#ifdef TIFFTAG_NUMBEROFINKS
         if (!TIFFGetField(tif, TIFFTAG_INKSET, &inkset) &&
 	    !TIFFGetField(tif, TIFFTAG_NUMBEROFINKS, &numinks))
+#else
+        if (!TIFFGetField(tif, TIFFTAG_INKSET, &inkset))
+#endif /* TIFFTAG_NUMBEROFINKS */
 	{
-          fputs("ERROR: No inkset or number-of-inks tag in the file!\n", stderr);
-	  fclose(fp);
-	  return (-1);
+          fputs("WARNING: No inkset or number-of-inks tag in the file!\n", stderr);
 	}
 
 	if (inkset == INKSET_CMYK || numinks == 4)
@@ -1362,10 +1417,10 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
                   }
         	}
               }
-              else if (img->colorspace == IMAGE_CMYK)
+              else if (img->colorspace == CUPS_IMAGE_CMYK)
 	      {
 	        TIFFReadScanline(tif, scanline, row, 0);
-		ImagePutRow(img, 0, y, img->xsize, scanline);
+		_cupsImagePutRow(img, 0, y, img->xsize, scanline);
 	      }
 	      else
               {
@@ -1412,37 +1467,40 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
               }
 
               if ((saturation != 100 || hue != 0) && bpp > 1)
-        	ImageRGBAdjust(in, img->xsize, saturation, hue);
+        	cupsImageRGBAdjust(in, img->xsize, saturation, hue);
 
-              if (img->colorspace == IMAGE_RGB)
+              if (img->colorspace == CUPS_IMAGE_RGB)
 	      {
 	        if (lut)
-	          ImageLut(in, img->xsize * 3, lut);
+	          cupsImageLut(in, img->xsize * 3, lut);
 
-        	ImagePutRow(img, 0, y, img->xsize, in);
+        	_cupsImagePutRow(img, 0, y, img->xsize, in);
 	      }
-              else
+              else if (img->colorspace == CUPS_IMAGE_WHITE)
               {
 		switch (img->colorspace)
 		{
-		  case IMAGE_WHITE :
-		      ImageRGBToWhite(in, out, img->xsize);
+		  default :
 		      break;
-		  case IMAGE_BLACK :
-		      ImageRGBToBlack(in, out, img->xsize);
+
+		  case CUPS_IMAGE_WHITE :
+		      cupsImageRGBToWhite(in, out, img->xsize);
 		      break;
-		  case IMAGE_CMY :
-		      ImageRGBToCMY(in, out, img->xsize);
+		  case CUPS_IMAGE_BLACK :
+		      cupsImageRGBToBlack(in, out, img->xsize);
 		      break;
-		  case IMAGE_CMYK :
-		      ImageRGBToCMYK(in, out, img->xsize);
+		  case CUPS_IMAGE_CMY :
+		      cupsImageRGBToCMY(in, out, img->xsize);
+		      break;
+		  case CUPS_IMAGE_CMYK :
+		      cupsImageRGBToCMYK(in, out, img->xsize);
 		      break;
 		}
 
 		if (lut)
-	          ImageLut(out, img->xsize * 3, lut);
+	          cupsImageLut(out, img->xsize * 3, lut);
 
-        	ImagePutRow(img, 0, y, img->xsize, out);
+        	_cupsImagePutRow(img, 0, y, img->xsize, out);
 	      }
             }
           }
@@ -1588,10 +1646,10 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
                   }
         	}
               }
-              else if (img->colorspace == IMAGE_CMYK)
+              else if (img->colorspace == CUPS_IMAGE_CMYK)
 	      {
 	        TIFFReadScanline(tif, scanline, row, 0);
-		ImagePutCol(img, x, 0, img->ysize, scanline);
+		_cupsImagePutCol(img, x, 0, img->ysize, scanline);
 	      }
               else
               {
@@ -1638,37 +1696,40 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
               }
 
               if ((saturation != 100 || hue != 0) && bpp > 1)
-        	ImageRGBAdjust(in, img->ysize, saturation, hue);
+        	cupsImageRGBAdjust(in, img->ysize, saturation, hue);
 
-              if (img->colorspace == IMAGE_RGB)
+              if (img->colorspace == CUPS_IMAGE_RGB)
 	      {
 		if (lut)
-	          ImageLut(in, img->ysize * 3, lut);
+	          cupsImageLut(in, img->ysize * 3, lut);
 
-        	ImagePutCol(img, x, 0, img->ysize, in);
+        	_cupsImagePutCol(img, x, 0, img->ysize, in);
               }
-	      else
+              else if (img->colorspace == CUPS_IMAGE_WHITE)
               {
 		switch (img->colorspace)
 		{
-		  case IMAGE_WHITE :
-		      ImageRGBToWhite(in, out, img->ysize);
+		  default :
 		      break;
-		  case IMAGE_BLACK :
-		      ImageRGBToBlack(in, out, img->ysize);
+
+		  case CUPS_IMAGE_WHITE :
+		      cupsImageRGBToWhite(in, out, img->ysize);
 		      break;
-		  case IMAGE_CMY :
-		      ImageRGBToCMY(in, out, img->ysize);
+		  case CUPS_IMAGE_BLACK :
+		      cupsImageRGBToBlack(in, out, img->ysize);
 		      break;
-		  case IMAGE_CMYK :
-		      ImageRGBToCMYK(in, out, img->ysize);
+		  case CUPS_IMAGE_CMY :
+		      cupsImageRGBToCMY(in, out, img->ysize);
+		      break;
+		  case CUPS_IMAGE_CMYK :
+		      cupsImageRGBToCMYK(in, out, img->ysize);
 		      break;
 		}
 
 		if (lut)
-	          ImageLut(out, img->ysize * bpp, lut);
+	          cupsImageLut(out, img->ysize * bpp, lut);
 
-        	ImagePutCol(img, x, 0, img->ysize, out);
+        	_cupsImagePutCol(img, x, 0, img->ysize, out);
 	      }
             }
           }
@@ -1697,11 +1758,9 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
   TIFFClose(tif);
   return (0);
 }
-
-
 #endif /* HAVE_LIBTIFF */
 
 
 /*
- * End of "$Id: image-tiff.c,v 1.20 2002/01/21 22:25:46 mike Exp $".
+ * End of "$Id: image-tiff.c 4741 2005-10-02 04:25:52Z mike $".
  */
