@@ -1,9 +1,9 @@
 /*
- * "$Id: lpoptions.c,v 1.11 2002/02/12 18:47:13 mike Exp $"
+ * "$Id: lpoptions.c 4924 2006-01-13 01:55:20Z mike $"
  *
  *   Printer option program for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -33,9 +33,11 @@
  * Include necessary headers...
  */
 
-#include <cups/cups.h>
 #include <cups/string.h>
+#include <cups/cups.h>
+#include <cups/i18n.h>
 #include <stdlib.h>
+#include <errno.h>
 
 
 /*
@@ -51,20 +53,20 @@ void	usage(void);
  * 'main()' - Main entry.
  */
 
-int				/* O - Exit status */
-main(int  argc,			/* I - Number of command-line arguments */
-     char *argv[])		/* I - Command-line arguments */
+int					/* O - Exit status */
+main(int  argc,				/* I - Number of command-line arguments */
+     char *argv[])			/* I - Command-line arguments */
 {
-  int		i, j;		/* Looping vars */
-  int		changes;	/* Did we make changes? */
-  int		num_options;	/* Number of options */
-  cups_option_t	*options;	/* Options */
-  int		num_dests;	/* Number of destinations */
-  cups_dest_t	*dests;		/* Destinations */
-  cups_dest_t	*dest;		/* Current destination */
-  char		*printer,	/* Printer name */
-		*instance,	/* Instance name */ 
- 		*option;	/* Current option */
+  int		i, j;			/* Looping vars */
+  int		changes;		/* Did we make changes? */
+  int		num_options;		/* Number of options */
+  cups_option_t	*options;		/* Options */
+  int		num_dests;		/* Number of destinations */
+  cups_dest_t	*dests;			/* Destinations */
+  cups_dest_t	*dest;			/* Current destination */
+  char		*printer,		/* Printer name */
+		*instance,		/* Instance name */ 
+ 		*option;		/* Current option */
 
 
  /*
@@ -103,7 +105,8 @@ main(int  argc,			/* I - Number of command-line arguments */
 
             if ((dest = cupsGetDest(printer, instance, num_dests, dests)) == NULL)
 	    {
-	      fputs("lpoptions: Unknown printer or class!\n", stderr);
+	      _cupsLangPuts(stderr,
+	                    _("lpoptions: Unknown printer or class!\n"));
 	      return (1);
 	    }
 
@@ -119,9 +122,10 @@ main(int  argc,			/* I - Number of command-line arguments */
 	    cupsSetDests(num_dests, dests);
 
 	    for (j = 0; j < dest->num_options; j ++)
-	      num_options = cupsAddOption(dest->options[j].name,
-	                                  dest->options[j].value,
-	                                  num_options, &options);
+	      if (cupsGetOption(dest->options[j].name, num_options, options) == NULL)
+		num_options = cupsAddOption(dest->options[j].name,
+	                                    dest->options[j].value,
+	                                    num_options, &options);
 	    break;
 
 	case 'h' : /* -h server */
@@ -137,22 +141,22 @@ main(int  argc,			/* I - Number of command-line arguments */
 	    }
 	    break;
 
+        case 'E' : /* Encrypt connection */
+	    cupsSetEncryption(HTTP_ENCRYPT_REQUIRED);
+	    break;
+
 	case 'l' : /* -l (list options) */
             if (dest == NULL)
 	    {
 	      if (num_dests == 0)
 		num_dests = cupsGetDests(&dests);
 
-	      for (i = num_dests, dest = dests; i > 0; i --, dest ++)
-	        if (dest->is_default)
-		  break;
-
-              if (i == 0)
+	      if ((dest = cupsGetDest(NULL, NULL, num_dests, dests)) == NULL)
 	        dest = dests;
 	    }
 
             if (dest == NULL)
-	      fputs("lpoptions: No printers!?!\n", stderr);
+	      _cupsLangPuts(stderr, _("lpoptions: No printers!?!\n"));
 	    else
 	      list_options(dest);
 
@@ -199,15 +203,19 @@ main(int  argc,			/* I - Number of command-line arguments */
 
               if (dest == NULL)
 	      {
-	        perror("lpoptions: Unable to add printer or instance");
+	        _cupsLangPrintf(stderr,
+		                _("lpoptions: Unable to add printer or "
+				  "instance: %s\n"),
+				strerror(errno));
 		return (1);
 	      }
 	    }
 
 	    for (j = 0; j < dest->num_options; j ++)
-	      num_options = cupsAddOption(dest->options[j].name,
-	                                  dest->options[j].value,
-	                                  num_options, &options);
+	      if (cupsGetOption(dest->options[j].name, num_options, options) == NULL)
+		num_options = cupsAddOption(dest->options[j].name,
+	                                    dest->options[j].value,
+	                                    num_options, &options);
 	    break;
 
 	case 'r' : /* -r option (remove) */
@@ -258,26 +266,34 @@ main(int  argc,			/* I - Number of command-line arguments */
 	    if (num_dests == 0)
 	      num_dests = cupsGetDests(&dests);
 
-            if ((dest = cupsGetDest(printer, instance, num_dests, dests)) == NULL)
+            if ((dest = cupsGetDest(printer, instance, num_dests, dests)) != NULL)
 	    {
-	      if (instance)
-		fprintf(stderr, "lpoptions: No such printer/instance: %s/%s\n",
-		        printer, instance);
-	      else
-		fprintf(stderr, "lpoptions: No such printer: %s\n", printer);
+              cupsFreeOptions(dest->num_options, dest->options);
 
-	      return (1);
+             /*
+	      * If we are "deleting" the default printer, then just set the
+	      * number of options to 0; if it is also the system default
+	      * then cupsSetDests() will remove it for us...
+	      */
+
+	      if (dest->is_default)
+	      {
+		dest->num_options = 0;
+		dest->options     = NULL;
+	      }
+	      else
+	      {
+		num_dests --;
+
+		j = dest - dests;
+		if (j < num_dests)
+		  memcpy(dest, dest + 1, (num_dests - j) * sizeof(cups_dest_t));
+	      }
 	    }
 
-            cupsFreeOptions(dest->num_options, dest->options);
-	    num_dests --;
-
-	    j = dest - dests;
-	    if (j < num_dests)
-	      memcpy(dest, dest + 1, (num_dests - j) * sizeof(cups_dest_t));
-
 	    cupsSetDests(num_dests, dests);
-	    dest = NULL;
+	    dest    = NULL;
+	    changes = -1;
 	    break;
 
 	default :
@@ -291,12 +307,16 @@ main(int  argc,			/* I - Number of command-line arguments */
     num_dests = cupsGetDests(&dests);
 
   if (dest == NULL)
-    for (i = 0; i < num_dests; i ++)
-      if (dests[i].is_default)
-      {
-        dest = dests + i;
-	break;
-      }
+  {
+    if ((dest = cupsGetDest(NULL, NULL, num_dests, dests)) != NULL)
+    {
+      for (j = 0; j < dest->num_options; j ++)
+	if (cupsGetOption(dest->options[j].name, num_options, options) == NULL)
+	  num_options = cupsAddOption(dest->options[j].name,
+	                              dest->options[j].value,
+	                              num_options, &options);
+    }
+  }
 
   if (dest == NULL)
     return (0);
@@ -322,18 +342,20 @@ main(int  argc,			/* I - Number of command-line arguments */
     for (i = 0; i < num_options; i ++)
     {
       if (i)
-        putchar(' ');
+        _cupsLangPuts(stdout, " ");
 
       if (!options[i].value[0])
-        printf("%s", options[i].name);
+        _cupsLangPrintf(stdout, "%s", options[i].name);
       else if (strchr(options[i].value, ' ') != NULL ||
                strchr(options[i].value, '\t') != NULL)
-	printf("%s=\'%s\'", options[i].name, options[i].value);
+	_cupsLangPrintf(stdout, "%s=\'%s\'", options[i].name,
+	                options[i].value);
       else
-	printf("%s=%s", options[i].name, options[i].value);
+	_cupsLangPrintf(stdout, "%s=%s", options[i].name,
+	                options[i].value);
     }
 
-    putchar('\n');
+    _cupsLangPuts(stdout, "\n");
   }
 
   return (0);
@@ -354,15 +376,15 @@ list_group(ppd_group_t *group)	/* I - Group to show */
 
   for (i = group->num_options, option = group->options; i > 0; i --, option ++)
   {
-    printf("%s/%s:", option->keyword, option->text);
+    _cupsLangPrintf(stdout, "%s/%s:", option->keyword, option->text);
 
     for (j = option->num_choices, choice = option->choices; j > 0; j --, choice ++)
       if (choice->marked)
-        printf(" *%s", choice->choice);
+        _cupsLangPrintf(stdout, " *%s", choice->choice);
       else
-        printf(" %s", choice->choice);
+        _cupsLangPrintf(stdout, " %s", choice->choice);
 
-    putchar('\n');
+    _cupsLangPuts(stdout, "\n");
   }
 
   for (i = group->num_subgroups, subgroup = group->subgroups; i > 0; i --, subgroup ++)
@@ -385,14 +407,18 @@ list_options(cups_dest_t *dest)	/* I - Destination to list */
 
   if ((filename = cupsGetPPD(dest->name)) == NULL)
   {
-    fprintf(stderr, "lpoptions: Destination %s has no PPD file!\n", dest->name);
+    _cupsLangPrintf(stderr,
+                    _("lpoptions: Destination %s has no PPD file!\n"),
+		    dest->name);
     return;
   }
 
   if ((ppd = ppdOpenFile(filename)) == NULL)
   {
     unlink(filename);
-    fprintf(stderr, "lpoptions: Unable to open PPD file for %s!\n", dest->name);
+    _cupsLangPrintf(stderr,
+                    _("lpoptions: Unable to open PPD file for %s!\n"),
+		    dest->name);
     return;
   }
 
@@ -414,15 +440,17 @@ list_options(cups_dest_t *dest)	/* I - Destination to list */
 void
 usage(void)
 {
-  puts("Usage: lpoptions -d printer");
-  puts("       lpoptions [-p printer] -l");
-  puts("       lpoptions -p printer -o option[=value] ...");
-  puts("       lpoptions -x printer");
+  _cupsLangPuts(stdout,
+                _("Usage: lpoptions [-h server] [-E] -d printer\n"
+		  "       lpoptions [-h server] [-E] [-p printer] -l\n"
+		  "       lpoptions [-h server] [-E] -p printer -o "
+		  "option[=value] ...\n"
+		  "       lpoptions [-h server] [-E] -x printer\n"));
 
   exit(1);
 }
 
 
 /*
- * End of "$Id: lpoptions.c,v 1.11 2002/02/12 18:47:13 mike Exp $".
+ * End of "$Id: lpoptions.c 4924 2006-01-13 01:55:20Z mike $".
  */
